//
//  BlackWhiteFilterViewController.m
//  BlackWhiteFilter
//
//  Created by Heiko Kalista on 13.03.11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "BlackWhiteFilterViewController.h"


@implementation BlackWhiteFilterViewController

- (void)dealloc {
    [imageView release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
}
*/

- (void)viewDidUnload {
    [imageView release];
    imageView = nil;
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return YES;
}


- (IBAction)onButtonConvertClicked:(id)sender {
    
    NSArray *pathList = NSSearchPathForDirectoriesInDomains( NSDocumentDirectory, NSUserDomainMask, YES );
    NSString *documentsDirectory = [pathList objectAtIndex:0];
    NSString *destinationDirectory = [NSString stringWithFormat:@"%@/output", documentsDirectory];
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    
    // Ausgabeverzeichnis erzeugen, falls nicht vorhanden
    BOOL isDirectory = NO;
    
    if (([fileManager fileExistsAtPath:destinationDirectory isDirectory:&isDirectory] && isDirectory == YES) == NO) {
        [fileManager createDirectoryAtPath: destinationDirectory withIntermediateDirectories: NO attributes: nil error: NULL];
    }
    
    
    // Alle Dateien im Dokumentenverzeichnis auflisten
    NSArray *filelist;
    filelist = [fileManager contentsOfDirectoryAtPath:documentsDirectory error:NULL];
    
    // Die erhaltene Liste nach Bitmap-Dateien filtern
    NSArray *bmpFilter = [NSArray arrayWithObjects:@"bmp", nil];
    NSArray *bitmapFileList = [filelist filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"pathExtension IN %@", bmpFilter]];
    
    
    // Alle Bitmaps konvertieren
    int count = [bitmapFileList count];
    
    for (int i = 0; i < count; i++) {
        NSLog(@"%@", [bitmapFileList objectAtIndex: i]);
        
        NSString *pathOfSourceFile = [NSString stringWithFormat:@"%@/%@", documentsDirectory, [bitmapFileList objectAtIndex: i]];
        NSString *pathOfDestinationFile = [NSString stringWithFormat:@"%@/%@", destinationDirectory, [bitmapFileList objectAtIndex: i]];
        
        [self convertImage: pathOfSourceFile :pathOfDestinationFile];
    }      
}


- (void)convertImage:(NSString*)sourceFileName :(NSString*)destinationFileName {
    
    imageView.image = [[[UIImage alloc] initWithContentsOfFile:sourceFileName] autorelease];
    
       // Datei öffnen
	NSFileHandle *fileHandle = [NSFileHandle fileHandleForReadingAtPath:sourceFileName];
    
	if (fileHandle == NULL)	{
		NSLog(@"Failed to load: %@", sourceFileName);
		return;
	}
	else {
		NSLog(@"File %@ loaded successfully", sourceFileName);
	}
    
    
    // Bitmap-Header auslesen
    struct BitmapHeader bitmapHeader;
    
    int bitmapHeaderSize = sizeof(bitmapHeader);
    
    NSData *bitmapHeaderData = [fileHandle readDataOfLength:bitmapHeaderSize];
    
    [bitmapHeaderData getBytes:&bitmapHeader length:bitmapHeaderSize];
    
    
    // Pixeldaten auslesen
    NSData *bitmapPixelData = [fileHandle readDataOfLength:bitmapHeader.bitmapDataSize];

    Byte *pixelData = (Byte*)malloc(bitmapHeader.bitmapDataSize);
    [bitmapPixelData getBytes:pixelData length:bitmapHeader.bitmapDataSize];
    
    [fileHandle closeFile];
    
    int stride = (bitmapHeader.width * (bitmapHeader.bpp / 8) + 3) & ~3;
    
    
    // Graustufenkonvertierung
    int numBytes = bitmapHeader.bpp/8;
    
    
    for (int y=0; y<bitmapHeader.height; y++) {
        int offset = y*stride;
        
        for (int x=0; x<bitmapHeader.width; x++) {
            float greyValue = pixelData[offset+x*numBytes+0]*0.1140f +
            pixelData[offset+x*numBytes+1]*0.5870f +
            pixelData[offset+x*numBytes+2]*0.2989;
            
            
            pixelData[offset+x*numBytes+0] = greyValue;
            pixelData[offset+x*numBytes+1] = greyValue;
            pixelData[offset+x*numBytes+2] = greyValue;
        }
    }
    
    bitmapPixelData = [NSData dataWithBytes:(const void *)pixelData length:bitmapHeader.bitmapDataSize];
          
    
    // Datei schreiben
    fileHandle = [NSFileHandle fileHandleForWritingAtPath:destinationFileName];
    
    if(fileHandle == nil) {
        [[NSFileManager defaultManager] createFileAtPath:destinationFileName contents:nil attributes:nil];
        fileHandle = [NSFileHandle fileHandleForWritingAtPath:destinationFileName];
    }    
    
    
    if(fileHandle != nil) {
        [fileHandle writeData:bitmapHeaderData];
        [fileHandle writeData:bitmapPixelData];
        
        [fileHandle closeFile];    
    }
    
    free(pixelData);
}

@end


